@@ -2,7 +2,6 @@ package t::lib::XSP::Test;
 
 use strict;
 use warnings;
-use lib 't/lib';
 use if -d 'blib' => 'blib';
 
 use Test::Base -Base;
@@ -10,6 +9,17 @@ use Test::Differences;
 
 our @EXPORT = qw(run_diff);
 
+# allows running tests both from t and from the top directory
+use File::Spec;
+BEGIN {
+  if (-d 't') {
+    unshift @INC, File::Spec->catdir(qw(t lib));
+  }
+  elsif (-d "lib") {
+    unshift @INC, "lib";
+  }
+}
+
 filters { xsp_stdout => 'xsp_stdout',
           xsp_file   => 'xsp_file',
           };
@@ -31,7 +41,18 @@ package t::lib::XSP::Test::Filter;
 
 use Test::Base::Filter -base;
 
+no warnings 'redefine';
+
+# some fixed "random" values to simplify testing
+my( @random_list, @random_digits ) =
+    qw(017082 074990 737474 643532 738748 630394 284033);
+sub ExtUtils::XSpp::Grammar::_random_digits {
+    die "No more random values" unless @random_digits;
+    return shift @random_digits;
+}
+
 sub xsp_stdout {
+    @random_digits = @random_list;
     my $d = ExtUtils::XSpp::Driver->new( string => shift );
     my $out = $d->generate;
 
@@ -39,6 +60,7 @@ sub xsp_stdout {
 }
 
 sub xsp_file {
+    @random_digits = @random_list;
     my $name = Test::Base::filter_arguments();
     my $d = ExtUtils::XSpp::Driver->new( string => shift );
     my $out = $d->generate;
